@@ -22,21 +22,26 @@ int main(int argc, char *argv[])
     agency = MC_Initialize(
             5051,
             &options);
-
+	MC_ResetSignal(agency);
     /* Retrieve the first arriving agent */
     /* Note: MC_MainLoopRetrieveAgent() pauses the agency: We'll need to unpause 
      * it later with MC_SignalReset() */
     agent = MC_WaitRetrieveAgent(agency);
-    printf("The agent status is: %d\n", MC_GetAgentStatus(agent));
-    printf("This agent has %d task(s).\n", MC_GetAgentNumTasks(agent));
-    str = MC_GetAgentXMLString(agent);
-    printf("Agent XML String:\n%s\n", str);
-    free(str);
-    str = MC_RetrieveAgentCode(agent);
-    printf("Agent Code:\n%s\n", str);
-    free(str);
-    MC_ResetSignal(agency);
-    MC_MainLoop(agency);
+	if (agent != NULL)
+	{
+		printf("The agent status is: %d\n", MC_GetAgentStatus(agent));
+		printf("This agent has %d task(s).\n", MC_GetAgentNumTasks(agent));
+		str = MC_GetAgentXMLString(agent);
+		printf("Agent XML String:\n%s\n", str);
+		free(str);
+		str = MC_RetrieveAgentCode(agent);
+		printf("Agent Code:\n%s\n", str);
+		free(str);
+		MC_ResetSignal(agency);
+		MC_MainLoop(agency);
+	}
+	else
+		printf("Error: returned NULL pointer for agent.\n");
 
     return 0;
 }
