@@ -55,6 +55,7 @@
 #include "include/message.h"
 #include "include/mtp_http.h"
 #include "include/xml_parser.h"
+#include "include/fipa_acl_envelope.h"
 
 #define BACKLOG 10
 
@@ -65,6 +66,12 @@ acc_Initialize(struct mc_platform_s* mc_platform)
   acc = (acc_p)malloc(sizeof(acc_t));
   acc->mc_platform = mc_platform;
 
+  acc->waiting = 0;
+  acc->waiting_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
+  MUTEX_INIT(acc->waiting_lock);
+  acc->waiting_cond = (COND_T*)malloc(sizeof(COND_T));
+  COND_INIT(acc->waiting_cond);
+
   return acc;
 }
 
@@ -91,6 +98,9 @@ acc_MessageHandlerThread( LPVOID arg )
   message_p message;
   agent_p agent;
   int mobile_agent_counter = 1;
+  char* tmpstr;
+  char* origname;
+  int i;
 
   while(1) 
   {
@@ -147,6 +157,22 @@ acc_MessageHandlerThread( LPVOID arg )
              mobile_agent_counter
             );
           if (agent != NULL) {
+            /* We need to make sure there are no agent-name collisions. */
+            i = 1;
+            if( agent_queue_SearchName( mc_platform->agent_queue, agent->name ) ) {
+              origname = agent->name;
+              while( agent_queue_SearchName( mc_platform->agent_queue, agent->name ) ) {
+                /* An agent with a matching name was found! For now, let us just
+                 * rename the agent. */
+                tmpstr = (char*)malloc(sizeof(char)* strlen(origname)+5);
+                sprintf(tmpstr, "%s_%04d", origname, i);
+                agent->name = tmpstr;
+                i++;
+              }
+              fprintf(stderr, "Warning: Agent '%s' has been renamed to '%s'.\n",
+                  origname, agent->name);
+              free(origname);
+            }
             mobile_agent_counter++;
             agent_queue_Add
               (
@@ -175,7 +201,8 @@ acc_MessageHandlerThread( LPVOID arg )
           MUTEX_UNLOCK(mc_platform->ams->runflag_lock);
           break;
         case FIPA_ACL:
-          /* FIXME: Process ACL Message Here */
+          /* This case should never happen now, due to a reorganization
+           * of MobileC. This enum will be deprecated in a future release. */
           break;
         case RETURN_MSG:
           /* Add the persistent neutral agent. */
@@ -321,6 +348,12 @@ acc_Thread( LPVOID arg )
   message_p message;
   mtp_http_p mtp_http;
   mc_platform_p mc_platform = (mc_platform_p)arg;
+  fipa_acl_envelope_p fipa_envelope;
+  fipa_acl_message_p fipa_message;
+  fipa_message_string_p fipa_message_string;
+  int err;
+  int i, j;
+  agent_t* agent;
   /* We want to watch the connectlist for incoming connections, initialize 
    * them into messages, and add the messages to the appropriate queue. */
   while(1) {
@@ -375,20 +408,24 @@ acc_Thread( LPVOID arg )
       continue;
     }
 
-    /* See if it is a message for the AMS */
     switch(mtp_http->http_performative)
     {
       case HTTP_POST:
       case HTTP_PUT:
         /* See if the incoming message is going to any of our services. */
-        if(!strcmp(mtp_http->target, "/ams")) {
+        /* See if it is a message for the AMS */
+        if(
+            !strcmp(mtp_http->target, "/ams") ||
+            !strcmp( strrchr(mtp_http->target, (int)'/'), "/ams" )
+            ) {
           message = message_New();
-          message->message_body = malloc
+          /*message->message_parts = mtp_http->message_parts;*/
+          message->message_body = (char*)malloc
             (
              sizeof(char) * 
-             (strlen((char*)mtp_http->data)+1)
+             (strlen((char*)mtp_http->content->data)+1)
             );
-          strcpy(message->message_body, mtp_http->data);
+          strcpy(message->message_body, (char*)mtp_http->content->data);
           message->xml_root = mxmlLoadString
             (
              NULL,
@@ -402,8 +439,63 @@ acc_Thread( LPVOID arg )
             mtp_http_Destroy(mtp_http);
             continue;
           }
+          mtp_http_Destroy(mtp_http);
           break;
-        } else {
+        } else if 
+          ( 
+           !strcmp(mtp_http->target, "/acc") ||
+           !strcmp( strrchr(mtp_http->target, (int)'/'), "/acc")
+          ) {
+          /* Make sure there are two parts to the html message.
+           * We expect to receive an xml envelope with a message attached. */
+          if (mtp_http->message_parts != 2) {
+            fprintf(stderr, "Error parsing message. %s:%d\n",
+                __FILE__,__LINE__);
+            mtp_http_Destroy(mtp_http);
+            continue;
+          }
+          /* Now we need to parse the envelope */
+          fipa_envelope = fipa_acl_envelope_New();
+          err = fipa_envelope_Parse(fipa_envelope, (char*)mtp_http->content[0].data);
+          if (err) {
+            fprintf(stderr, "Error parsing message. %s:%d\n",
+                __FILE__, __LINE__);
+            fipa_acl_envelope_Destroy(fipa_envelope);
+            mtp_http_Destroy(mtp_http);
+            continue;
+          }
+          /* Now we need to check the envelope receivers and put copies of the message into
+           * each receiving agent's mailbox. */
+          for(i = 0; i < fipa_envelope->num_params; i++) {
+            for(j = 0; j < fipa_envelope->params[i]->to->num; j++) {
+              agent = agent_queue_SearchName(
+                  mc_platform->agent_queue,
+                  fipa_envelope->params[i]->to->fipa_agent_identifiers[j]->name
+                  );
+              if (agent != NULL) {
+                /* Parse, copy, and deliver the message to the agent. */
+                fipa_message_string = fipa_message_string_New();
+                fipa_message_string->message = strdup((char*)mtp_http->content[1].data);
+                fipa_message_string->parse = fipa_message_string->message;
+                fipa_message = fipa_acl_message_New();
+                err = fipa_acl_Parse(fipa_message, fipa_message_string);
+                if (err) {
+                  fipa_message_string_Destroy(fipa_message_string);
+                  fipa_acl_message_Destroy(fipa_message);
+                  fipa_acl_envelope_Destroy(fipa_envelope);
+                  mtp_http_Destroy(mtp_http);
+                  continue;
+                }
+                agent_mailbox_Post( agent->mailbox, fipa_message);
+                fipa_message_string_Destroy(fipa_message_string);
+              }
+            }
+          }
+          fipa_acl_envelope_Destroy(fipa_envelope);
+          mtp_http_Destroy(mtp_http);
+          continue;
+        }
+        else {
           /* FIXME: We don't support other recipients yet */
           fprintf(stderr, "Unsupported. %s:%d\n", __FILE__, __LINE__);
           mtp_http_Destroy(mtp_http);
@@ -548,6 +640,11 @@ listen_Thread( LPVOID arg )
   /* this while loop runs continuously creating connections */
   while(1)
   {
+    /* Set waiting flag */
+    MUTEX_LOCK(mc_platform->acc->waiting_lock);
+    mc_platform->acc->waiting = 1;
+    COND_BROADCAST(mc_platform->acc->waiting_cond);
+    MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
 #ifndef _WIN32
     if((connectionsockfd = accept(sockfd, 
             (struct sockaddr *)&peer_addr, 
@@ -567,6 +664,11 @@ listen_Thread( LPVOID arg )
       }
       else 
       {
+        /* Set waiting flag */
+        MUTEX_LOCK(mc_platform->acc->waiting_lock);
+        mc_platform->acc->waiting = 0;
+        COND_BROADCAST(mc_platform->acc->waiting_cond);
+        MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
         /* create a new connection and insert it into the connection linked 
          * list */
         connection = (connection_p)malloc(sizeof(connection_t));
