@@ -29,10 +29,26 @@
  * to the extent permitted by applicable law.
 ]*/
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include "include/agent_task.h"
 #include "include/mc_error.h"
 
+agent_task_p
+agent_task_New(void)
+{
+  agent_task_p task;
+  task = (agent_task_p)malloc(sizeof(agent_task_t));
+  if(task == NULL) {
+    fprintf(stderr, "Memory Error. %s:%d\n", __FILE__, __LINE__);
+  } else {
+    memset(task, 0, sizeof(agent_task_t)); 
+  }
+
+  return task;
+}
+
 agent_task_p 
 agent_task_Copy(agent_task_p task)
 {
