@@ -40,6 +40,12 @@ STRUCT
   MUTEX_T *runflag_lock;
   COND_T *runflag_cond;
   int run;
+
+  /* waiting flag */
+  int waiting;
+  MUTEX_T* waiting_lock;
+  COND_T* waiting_cond;
+
   THREAD_T thread;
   )
 
