@@ -36,6 +36,10 @@
 struct mc_platform_s;
 /* Connection Queue */
 #include "connection.h"
+/* Fipa mailbox */
+#include "fipa_acl.h"
+/* Agent Mailbox */
+#include "agent_mailbox.h"
 AP_QUEUE_DECL_TEMPLATE(
     connection_queue, /* name */
     connection /* node_type */
@@ -105,4 +109,52 @@ AP_QUEUE_GENERIC_DECL_TEMPLATE(
 
 int
 agent_queue_Print(agent_queue_p queue);
+
+/* Mail Queue */
+AP_QUEUE_DECL_TEMPLATE(
+    mail_queue,
+    fipa_acl_message
+    )
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    mail_queue,
+    SearchReceivers,
+    fipa_acl_message_p,
+    char*
+    )
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    mail_queue,
+    RemoveName,
+    int,
+    char*
+    )
+
+/* Mailbox Queue */
+AP_QUEUE_DECL_TEMPLATE(
+    mailbox_queue,
+    agent_mailbox
+    )
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    mailbox_queue,
+    SearchName,
+    agent_mailbox_p,
+    char*
+    )
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    mailbox_queue,
+    SearchID,
+    agent_mailbox_p,
+    int
+    )
+
+AP_QUEUE_GENERIC_DECL_TEMPLATE(
+    mailbox_queue,
+    RemoveID,
+    int,
+    int
+    )
+
 #endif
