@@ -162,6 +162,10 @@ typedef name##_t* name##_p;
 fprintf(stderr, "pthread lock error: %s:%d\n", __FILE__, __LINE__)
 #define MUTEX_UNLOCK(mutex) \
   pthread_mutex_unlock( mutex )
+#define MUTEX_NEW(mutex) \
+  mutex = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t)); \
+  if (mutex == NULL)  \
+    fprintf(stderr, "Memory Error. %s:%d\n", __FILE__,__LINE__); \
 
 /* **** *
  * COND *
@@ -329,6 +333,10 @@ pthread_cond_wait( queue->touched_signal, queue->thread_mutex )
       INFINITE)
 #define MUTEX_UNLOCK(mutex)         \
   ReleaseMutex( *mutex )
+#define MUTEX_NEW(mutex) \
+  mutex = (HANDLE*)malloc(sizeof(HANDLE)); \
+  if(mutex == NULL) \
+    fprintf(stderr, "Memory Error. %s:%d\n", __FILE__, __LINE__)
 
 
 /* **** *
@@ -375,7 +383,7 @@ SetEvent( *cond )
 #define SEMAPHORE_T HANDLE
 /* Init */
 #define SEMAPHORE_INIT(sem) \
-  sem = CreateSemaphore( \
+  *sem = CreateSemaphore( \
       NULL, \
       0, \
       1024, \
