@@ -75,21 +75,29 @@ typedef struct message_s{
   int connect_id;
   int message_id;
 
+  /* Flag if it is an http message already */
+  int isHTTP;
+
   /* the type of the message as discerned from the http header */
   enum message_type_e message_type;
   enum http_performative_e http_type;
 
-  mxml_node_t *xml_root;
-  mxml_node_t *xml_payload;
+  mxml_node_t* xml_root;
+  mxml_node_t* xml_payload;
 
   /* the http header and the rest of the message */
-  char *message_body;
+  char* message_body;
 
   char *update_name;
   int update_num;
 
-  char *from_address;
-  char *to_address;
+  char* from_address;
+  char* to_address;
+
+  char* target; /* This is the agency module to send the message to. It
+                   will either be 'ams' for migration messages, or 'acc'
+                   for ACL messages. ( And perhaps things like 'df' in the 
+                   future) */
 
   /* This flag is set if an agent is using the xml nodes, so we know
    * not to free/delete them prematurely. */
@@ -129,11 +137,15 @@ message_InitializeFromString(
     message_p message,
     const char* string,
     const char* destination_host,
-    int destination_port);
+    int destination_port,
+    const char* target);
 
 int 
 message_Destroy(message_p message);
 
 int 
 message_Send(message_p message);
+
+int 
+http_to_hostport(const char* http_str, char** host, int* port, char** target);
 #endif
