@@ -99,6 +99,12 @@ enum http_performative_e
   HTTP_NUM_PERFORMATIVES
 };
 
+typedef struct mtp_http_content_s
+{
+  char* content_type;
+  void* data;
+} mtp_http_content_t;
+
 typedef struct mtp_http_s
 {
   enum http_status_code_e http_status_code;
@@ -116,7 +122,14 @@ typedef struct mtp_http_s
   char* content_type;
   char* user_agent;
 
-  void* data;
+  char* cache_control;
+  char* mime_version;
+
+  /* The message may be a multi-part message */
+  /* FIXME: We don't support nested multi-part messages for now */
+  int message_parts;
+  char* boundary;
+  struct mtp_http_content_s* content;
 } mtp_http_t;
 typedef mtp_http_t* mtp_http_p;
 
@@ -175,4 +188,10 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string);
 struct message_s;
 int 
 mtp_http_ComposeMessage(struct message_s* message);
+
+struct message_s* 
+mtp_http_CreateMessage(
+    mtp_http_t* mtp_http,
+    char* hostname,
+    int port );
 #endif
