@@ -287,6 +287,7 @@ message_New(void)
   message->addr = NULL;
   message->connect_id = 0;
   message->message_id = 0;
+  message->isHTTP = 0;
   message->message_type = 0;
   message->http_type = 0;
   message->xml_root = NULL;
@@ -296,10 +297,12 @@ message_New(void)
   message->update_num = 0;
   message->from_address = NULL;
   message->to_address = NULL;
+  message->target = NULL;
   message->agent_xml_flag = 0;
   return message;
 }
 
+int
 message_InitializeFromAgent(
     mc_platform_p mc_platform,
     message_p message,
@@ -375,6 +378,7 @@ message_InitializeFromAgent(
         );
   }
   message->agent_xml_flag = 0;
+  message->target = strdup("ams");
   /* Set up message->addr */
   buf = (char*)malloc
     (
@@ -418,6 +422,7 @@ message_InitializeFromConnection(
 
   message->to_address = NULL;
   message->from_address = NULL;
+  message->target = NULL;
 
   buffer = (char*) malloc(sizeof(char) * (SOCKET_INPUT_SIZE + 1));
   CHECK_NULL(buffer, exit(0););
@@ -486,13 +491,48 @@ message_InitializeFromConnection(
   return MC_SUCCESS;  
 }
 
+int http_to_hostport(const char* http_str, char** host, int* port, char** target)
+{
+  /* We want to convert the string "http://somehost.com:5050/acc" to a 
+   * host: somehost.com
+   * port: 5050
+   * target: acc */
+  char* tmp;
+  if(strncmp(http_str, "http://", 7)) {
+    return MC_ERR_PARSE;
+  }
+  http_str += 7;
+  tmp = strchr(http_str, (int)':');
+  if (tmp == NULL) return MC_ERR_PARSE;
+
+  /* Get the host */
+  *host = (char*)malloc(sizeof(char) * 
+      (tmp - http_str + 1) );
+  strncpy(*host, http_str, tmp - http_str);
+  (*host)[tmp-http_str] = '\0';
+
+  /* Get the port */
+  tmp++;
+  sscanf(tmp, "%d", port);
+
+  /* Get the target */
+  tmp = strchr(tmp, (int)'/');
+  tmp++;
+  *target = (char*)malloc(sizeof(char) * 
+      (strlen(tmp)+1) );
+  strcpy(*target, tmp);
+
+  return 0;
+}
+
 int
 message_InitializeFromString(
     mc_platform_p mc_platform,
     message_p message,
     const char* string,
     const char* destination_host,
-    int destination_port )
+    int destination_port,
+    const char* target)
 {
   char* destination;
   struct hostent* host;
@@ -529,6 +569,9 @@ message_InitializeFromString(
       "%s:%d",
       mc_platform->hostname,
       mc_platform->port );
+  message->target = (char*)malloc(sizeof(char) * 
+      (strlen(target)+1));
+  strcpy(message->target, target);
 
   /* Set up message->addr */
   message->addr = (struct sockaddr_in*)malloc(sizeof(struct sockaddr_in));
@@ -547,6 +590,7 @@ message_InitializeFromString(
   int
 message_Destroy(message_p message)
 {
+  int i;
   if (message == NULL) {
     return MC_SUCCESS;
   }
@@ -573,6 +617,9 @@ message_Destroy(message_p message)
   if(message->to_address != NULL) {
     free(message->to_address);
   }
+  if(message->target != NULL) {
+    free(message->target);
+  }
 
   free(message);
   message = NULL;
