@@ -32,6 +32,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #ifndef _WIN32
 #include "config.h"
 #else
@@ -42,11 +43,13 @@
 #include "include/macros.h"
 #include "include/mc_error.h"
 #include "include/message.h"
+#include "include/dynstring.h"
 
 #define SOCKET_INPUT_SIZE 4096
 int
 mtp_http_Destroy(mtp_http_p http)
 {
+  int i;
 #define SAFE_FREE(elem) \
   if(elem) \
     free(elem)
@@ -61,16 +64,28 @@ mtp_http_Destroy(mtp_http_p http)
   SAFE_FREE(http->content_length);
   SAFE_FREE(http->connection);
   SAFE_FREE(http->content_type);
+  SAFE_FREE(http->user_agent);
+  if(http->content != NULL) {
+    for(i = 0; i < http->message_parts; i++) {
+      SAFE_FREE(http->content[i].content_type);
+      SAFE_FREE(http->content[i].data);
+    }
+  }
+  SAFE_FREE(http->content);
+  SAFE_FREE(http->boundary);
+  SAFE_FREE(http);
 #undef SAFE_FREE
   return 0;
 }
-mtp_http_p
+
+  mtp_http_p
 mtp_http_New(void)
 {
   mtp_http_p http;
   http = (mtp_http_p)malloc(sizeof(mtp_http_t));
   CHECK_NULL(http, exit(0););
   memset(http, 0, sizeof(mtp_http_t));
+  http->content = NULL;
   return http;
 }
 
@@ -86,8 +101,6 @@ mtp_http_InitializeFromConnection
   char *message_string;
   char *buffer;
   int message_size = 0;
-  int expected_content_size = -1;
-  int content_size = 0;
 
   buffer = (char*) malloc(sizeof(char) * (SOCKET_INPUT_SIZE + 1));
   CHECK_NULL(buffer, exit(0););
@@ -179,14 +192,31 @@ const char* http_GetExpression(const char* string, char** expr)
   int j;
   int success_flag = 0;
   const char* next_expr_ptr;
+  /* Check to see if we are at the end of the HTTP header */
+  if( 
+      (string[0] == '\n') ||
+      (string[0] == '\r' && string[1] == '\n')
+    ) 
+  {
+    for(i = 0; string[i] == '\n' || string[i] == '\r'; i++);
+    *expr = NULL;
+    return string+i;
+  }
+  /* FIXME */
   for(i = 0;string[i] != '\0';i++) {
     if (
         (
          (string[i] == '\r') &&
-         (string[i+1] == '\n')
+         (string[i+1] == '\n') &&
+         (string[i+2] != '\t') &&
+         (string[i+2] != ' ')
         ) 
         ||
-        (string[i] == '\n')
+        (
+         string[i] == '\n' && 
+         string[i+1] != '\t' &&
+         string[i+2] != ' '
+        )
        )
     {
       success_flag = 1;
@@ -204,14 +234,11 @@ const char* http_GetExpression(const char* string, char** expr)
     }
     (*expr)[j] = '\0';
     next_expr_ptr = &(string[i]);
-    while
-      (
-       (*next_expr_ptr == '\r') ||
-       (*next_expr_ptr == '\n')
-      )
-      {
-        next_expr_ptr++;
-      }
+    if(next_expr_ptr[0] == '\r' && next_expr_ptr[1] == '\n') {
+      next_expr_ptr += 2;
+    } else if (next_expr_ptr[0] == '\n') {
+      next_expr_ptr++;
+    }
     return next_expr_ptr;
   } else {
     return NULL;
@@ -227,8 +254,15 @@ int http_ParseExpression(
   int i=0;
   const char* tmp;
   const char* charptr;
+  if(expression_string == NULL) {
+    *name = NULL;
+    *value = NULL;
+    return MC_ERR_PARSE;
+  }
   tmp = expression_string;
-  if (tmp == NULL) {
+  if (tmp == NULL || (!strncmp(tmp, "\r\n", 2)) || (!strncmp(tmp, "\n", 1))) {
+    *name = NULL;
+    *value = NULL;
     return MC_ERR_PARSE;
   }
   for(; *tmp!=':' && *tmp!='\0'; tmp++)
@@ -277,6 +311,9 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
   char* expr = NULL;
   char* name = NULL;
   char* value = NULL;
+  char* tmp;
+  char* tmp2;
+  int i;
 
   int err_code = 0;
 
@@ -329,6 +366,8 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
     HTTP_PARSE_EXPR( "Connection", connection )
     HTTP_PARSE_EXPR( "Content-Type", content_type)
     HTTP_PARSE_EXPR( "User-Agent", user_agent)
+    HTTP_PARSE_EXPR( "Cache-Control", cache_control)
+    HTTP_PARSE_EXPR( "MIME-Version", mime_version)
     {
       fprintf(stderr, "Warning: Unknown http name: %s. %s:%d\n",
           name, __FILE__, __LINE__);
@@ -343,16 +382,126 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
     SAFE_FREE(name);
     SAFE_FREE(value);
 #undef SAFE_FREE
-      
+     
   } while(line != NULL && err_code == MC_SUCCESS);
-  /* Copy rest of contents into the data member */
-  if (line != NULL) {
-    http->data = (void*)malloc
-      (
-       sizeof(char) * 
-       (strlen(line)+1)
-      );
-    strcpy((char*)http->data, line);
+  /* If the content type is multipart/mixed, then we need to
+   * figure out how many parts there are. */
+  if( 
+      !strncmp(
+        http->content_type,
+        "multipart-mixed",
+        strlen("multipart-mixed")
+        ) 
+    ) 
+  {
+    tmp = strstr(http->content_type, "boundary=");
+    tmp += strlen("boundary=.");
+    tmp2 = strchr(tmp, '\"');
+    http->boundary = (char*)malloc(sizeof(char) * (tmp2 - tmp + 1));
+    for (i = 0; tmp != tmp2; i++, tmp++) {
+      http->boundary[i] = *tmp;
+    }
+    http->boundary[i] = '\0';
+
+    /* Count the number of message parts in the message */
+    tmp = (char*)line;
+    http->message_parts = 0;
+    while(tmp = strstr(tmp, http->boundary)) {
+      http->message_parts++;
+      tmp++;
+    }
+    http->message_parts--;
+  } else {
+    http->boundary = NULL;
+    http->message_parts = 1;
+  }
+
+  if (http->message_parts == 1) { 
+    http->content = (struct mtp_http_content_s*)malloc(sizeof(struct mtp_http_content_s));
+    /* Copy rest of contents into the data member */
+    if (line != NULL) {
+      http->content->data = (void*)malloc
+        (
+         sizeof(char) * 
+         (strlen(line)+1)
+        );
+      strcpy((char*)http->content->data, line);
+      http->content->content_type = strdup(http->content_type);
+    }
+  } else {
+    http->content = (struct mtp_http_content_s*)malloc(
+        sizeof(struct mtp_http_content_s) * http->message_parts );
+    memset(http->content, 0, sizeof(struct mtp_http_content_s) * http->message_parts);
+    /* Find the boundary */
+    line = strstr(line, http->boundary);
+    line += strlen(http->boundary);
+    line = strchr(line, '\n');
+    line++;
+    for(i = 0; i < http->message_parts; i++) {
+      /* For each part, we must:
+       * 1. Find the boundary
+       * 2. Parse the attributes, until we
+       * 3. Find the empty line
+       * 4. Copy the data up to the next boundary */
+      
+      /* Find the end boundary */
+      tmp = strstr(line + strlen(http->boundary), http->boundary);
+      /* Parse the attributes */
+      do{
+        /* FIXME */
+
+        line = http_GetExpression
+          (
+           line,
+           &expr
+          );
+
+        err_code = http_ParseExpression
+          (
+           expr,
+           &name,
+           &value
+          );
+        if
+          (
+           (name == NULL) ||
+           (value == NULL)
+          )
+          {
+            if (expr != NULL) {
+              free(expr);
+            }
+            break;
+          }
+        if (!strcmp(name, "Content-Type")) {
+          http->content[i].content_type = (char*)malloc(
+              sizeof(char) * (strlen(value)+1));
+          strcpy(http->content[i].content_type, value);
+        }
+
+        /* Clean up memory */
+        if (expr != NULL) {
+          free(expr);
+          expr = NULL;
+        }
+        if (name != NULL) {
+          free(name);
+          name = NULL;
+        }
+        if (value != NULL) {
+          free(value);
+          value = NULL;
+        }
+      } while(line != NULL && err_code == MC_SUCCESS);
+      /* Copy the data */
+      http->content[i].data = (void*)malloc(tmp-line+sizeof(char));
+      memcpy(http->content[i].data, line, tmp-line);
+      ((char*)http->content[i].data)[tmp-line] = '\0';
+      /* Move 'line' to the next boundary */
+      line = tmp + strlen(http->boundary);
+      line = strchr(line, '\n');
+      line++;
+    }
   }
   if (
       (http->http_performative == HTTP_POST) ||
@@ -370,6 +519,7 @@ const char* http_ParseHeader(
   const char* cur;
   char* token;
   char* tmp = NULL;
+  char* target;
 
   cur = string;
   cur = http_GetToken(cur, &token);
@@ -388,8 +538,15 @@ const char* http_ParseHeader(
     http->http_performative = HTTP_POST;
     cur = http_GetToken(cur, &tmp);
     if(tmp != NULL) {
-      http->target = (char*) malloc(sizeof(char) * (strlen(tmp)+1));
-      strcpy(http->target, tmp);
+      if(!strncmp(tmp, "http://",7)) {
+        target = strchr(tmp+7, (int)'/');
+      } else {
+        target = strchr(tmp, (int)'/');
+      }
+      if (target == NULL)
+        target = tmp;
+      http->target = (char*) malloc(sizeof(char) * (strlen(target)+1));
+      strcpy(http->target, target);
       free(tmp);
     }
   } else if(!strcmp(token, "PUT")) {
@@ -449,7 +606,7 @@ http_GetToken(const char* string, char** token)
       break;
   }
   cur--;
-  *token = (char*)malloc(cur - begin + 2*sizeof(char));
+  *token = (char*)malloc(cur - begin + 4*sizeof(char));
   itr = *token;
   while (begin <= cur) {
     *itr = *begin;
@@ -466,13 +623,19 @@ mtp_http_ComposeMessage(message_p message)
   char* http_header;
   char* tmp;
   char buf[20];
+  if (message->isHTTP) {
+    /* message reports that it already is an http message. No need to continue. */
+    return 0;
+  }
 
   http_header = (char*)malloc
     (
      sizeof(char) * (1400 + strlen(message->to_address))
     );
   http_header[0] = '\0';
-  strcat(http_header, "POST /ams HTTP/1.0\r\n");
+  strcat(http_header, "POST /");
+  strcat(http_header, message->target);
+  strcat(http_header, " HTTP/1.0\r\n");
   strcat(http_header, "User-Agent: MobileC/");
   strcat(http_header, PACKAGE_VERSION);
   strcat(http_header, "\r\n");
@@ -499,3 +662,80 @@ mtp_http_ComposeMessage(message_p message)
   free(http_header);
   return MC_SUCCESS;
 }
+
+struct message_s* 
+mtp_http_CreateMessage(
+    mtp_http_t* mtp_http,
+    char* hostname,
+    int port)
+{
+  int i;
+  int num;
+  char buf[30];
+  message_t* message;
+  dynstring_t* http_message;
+  http_message = dynstring_New();
+  dynstring_Append(http_message, "POST /");
+  dynstring_Append(http_message, mtp_http->target);
+  dynstring_Append(http_message, " HTTP/1.0\r\n");
+  dynstring_Append(http_message, "User-Agent: MobileC/");
+  dynstring_Append(http_message, PACKAGE_VERSION);
+  dynstring_Append(http_message, "\r\n");
+  dynstring_Append(http_message, "Host: ");
+  dynstring_Append(http_message, mtp_http->host);
+  dynstring_Append(http_message, "\r\n");
+
+  /* The next part of the message will be different depending on 
+   * how many message parts we have.  */
+  if(mtp_http->message_parts == 1) {
+    dynstring_Append(http_message, "Content-Type: text/plain\r\n");
+    dynstring_Append(http_message, "Connection: Close");
+    dynstring_Append(http_message, "Content-Length: ");
+    sprintf(buf, "%d", strlen((char*)mtp_http->content[0].data));
+    dynstring_Append(http_message, buf);
+    dynstring_Append(http_message, "\r\n\r\n");
+    dynstring_Append(http_message, (char*)mtp_http->content[0].data);
+  } else {
+    /* We need to generate a random boundary. */
+    srand(time(NULL));
+    strcpy(buf, "--");
+    for(i = 2; i < 26; i++) {
+      num = rand() % 36;
+      if(num < 10) {
+        buf[i] = (char)(48 + num);
+      } else {
+        buf[i] = (char)( (num-10)+65);
+      }
+    }
+    buf[i] = '\0';
+    /* FIXME: A randomly generated boundary should now be stored in buf. Should
+     * we check to see if there is a match within the body? The chances of collision are
+     * infinitessimal. */
+    dynstring_Append(http_message, "Content-Type: multipart-mixed ; boundary=\"");
+    dynstring_Append(http_message, buf);
+    dynstring_Append(http_message, "\"\r\n\r\n");
+    dynstring_Append(http_message, "This is not part of the MIME multipart encoded message.\r\n");
+    for(i = 0; i<mtp_http->message_parts; i++) {
+      dynstring_Append(http_message, buf);
+      dynstring_Append(http_message, "\nContent-Type: ");
+      dynstring_Append(http_message, mtp_http->content[i].content_type);
+      dynstring_Append(http_message, "\r\n\r\n");
+      dynstring_Append(http_message, (char*)mtp_http->content[i].data);
+      dynstring_Append(http_message, "\r\n\r\n");
+    }
+  }
+  /* Append one last boundary */
+  dynstring_Append(http_message, buf);
+  dynstring_Append(http_message, "\n");
+  message = message_New();
+  message->to_address = (char*)malloc(
+      sizeof(char) * (strlen(hostname)+15) );
+  sprintf(message->to_address, "%s:%d", hostname, port);
+  message->message_body = (char*) malloc( 
+      sizeof(char) * (http_message->len + 1));
+  strcpy(message->message_body, http_message->message);
+  dynstring_Destroy(http_message);
+  message->isHTTP = 1;
+  return message;
+}
+
