@@ -37,6 +37,11 @@ typedef struct mc_asm_s
   THREAD_T thread;
   asm_node_p home_encryption_info;
 
+  /* Waiting Flag */
+  int waiting;
+  MUTEX_T* waiting_lock;
+  COND_T* waiting_cond;
+
   /* The following queue is used to hold messages that have arrived
    * encrypted, but no information about the encryption could be attained.
    * The messages are checked when new encryption data arrives, to
